#    ECR Ion Source Control System

set debug 0
set started 0

set frame ecr
set frametitle "ECR Ion Source Control System"
set icontitle "ECRIS"

if {[info exists env(ECR_Server)]} then {set resource $env(ECR_Server)} else {set resource IP}

set testmode 0
set monitor 0

set autorefresh 0
set autosuspend 0

set currentDACchannel 1
set current16ADCchannel 1

set NumSamples 0

set hallprobe 0
set faradaycup 0

set hallprobevisible 0
set faradaycupvisible 0

set scanning 0
set ScanMode 0
set ScanModes [list Voltage Current Injection Middle Extraction]

set expertlist {                                              \
   {"Reload Preferences"                    "do-reload"}       \
   {"Edit Preferences"             "inform pedit start ECR"}  \
}

#    complete list of preferences and default values
set preferencelist [list \
             {List.Rows 5} {List.Cols 80} {Monitor.Samples 10} {Monitor.Period 100} {HallProbe.Trace.Width 500} \
             {HallProbe.Trace.Height 400} {HallProbe.Trace.Ymin -5000} {HallProbe.Trace.Ymax +5000} {HallProbe.Trace.Xrange 250} \
             {FaradayCup.Trace.Width 500} {FaradayCup.Trace.Height 400} {FaradayCup.Trace.Ymin -5000} {FaradayCup.Trace.Ymax +5000} \
             {FaradayCup.Trace.Xrange 250}  \
             {Scan.Current.Min 250} {Scan.Current.Max 950} {Scan.Current.Step 0.1} {Scan.Current.Settle 0.1} {Scan.Current.Init 10.0} \
             {Scan.Voltage.Min 20} {Scan.Voltage.Max 80} {Scan.Voltage.Step 0.01} {Scan.Voltage.Settle 0.1} {Scan.Voltage.Init 10.0} \
             {Scan.Injection.Min 250} {Scan.Injection.Max 950} {Scan.Injection.Step 0.1} {Scan.Injection.Settle 0.1} {Scan.Injection.Init 10.0} \
             {Scan.Middle.Min 20} {Scan.Middle.Max 80} {Scan.Middle.Step 0.01} {Scan.Middle.Settle 0.1} {Scan.Middle.Init 10.0} \
             {Scan.Extraction.Min 250} {Scan.Extraction.Max 950} {Scan.Extraction.Step 0.1} {Scan.Extraction.Settle 0.1} {Scan.Extraction.Init 10.0} \
             {Voltage.Min 0} {Voltage.Max 100} {Voltage.Resolution 0.01} {Voltage.p1 1} {Voltage.p2 0}  \
             {Current.Min 0} {Current.Max 1400} {Current.Resolution 0.1} {Current.p1 0.0321} {Current.p2 0.1919} \
             {Injection.Min 0} {Injection.Max 1200} {Injection.Resolution 0.1} {Injection.p1 0.0831} {Injection.p2 0.0405} \
             {Middle.Min 0} {Middle.Max 100} {Middle.Resolution 0.01} {Middle.p1 1} {Middle.p2 0} \
             {Extraction.Min 0} {Extraction.Max 1200} {Extraction.Resolution 0.1} {Extraction.p1 0.0831} {Extraction.p2 0.0354} \
             {Scan.SaveDirectory ""} \
            ]

loadsource ECR.defn

loadsource ipcommon


#    these file contain the IP register definitions
loadsource ECR.DAC
loadsource ECR.16ADC


proc initialise {} {

     define-IPs
     setup-IPs
     click-update
}

proc define-IPs {} {
    global IPmanID IPmodID IPslot

#    define all the IPs and check the configuration

#    IP-DAC
    define-DAC

    set manid [read-register ManID]
    set modid [read-register ModID]

    if {$manid != $IPmanID(IPDAC) && $modid != $IPmodID(IPDAC)} then {
        midas-warning "The IP in slot $IPslot(IPDAC) is not the correct type.\nAn IP-DAC module is required."
    } else {
        log "IP DAC in slot $IPslot(IPDAC) OK"
    }

#    IP-16ADC
    define-16ADC

    set manid [read-register ManID]
    set modid [read-register ModID]

    if {$manid != $IPmanID(IP16ADC) && $modid != $IPmodID(IP16ADC)} then {
        midas-warning "The IP in slot $IPslot(IP16ADC) is not the correct type.\nAn IP-16ADC module is required."
    } else {
        log "IP 16ADC in slot $IPslot(IP16ADC) OK"
    }
}

proc setup-IPs {} {
    global mname currentDACchannel current16ADCchannel

#    perform any initial setup required

#    IP-DAC
#    the DAC outputs 0 to +5V and so initialise all DAC channels to 0V

     set mname IPDAC
     set currentDACchannel 1
     for {set i 1} {$i <= 6} {incr i} {write-register c$i.DAC 0}

#    IP-16ADC
#    reset and setup the ADC; the ADC range is set to -5 to +5 V

     set mname IP16ADC

#    make SWRESET false and wait for A/D calibration cycle

     write-register Control 0x2178
     after 5000

     for {set i 1} {$i <= 8} {incr i} {setup_adc $i}

     set currentADCchannel 1
     select_adc_channel
}

proc select_adc_channel {} {
    global mname current16ADCchannel

    set mname IP16ADC

    set sel [expr ($current16ADCchannel - 1) + 0x18]
    set CSR [expr ([read-register Status] & 0xffe0) + $sel]
    write-register Control $CSR
}

proc setup_adc {ch} {
    global mname current16ADCchannel

    set mname IP16ADC

#    set as 8 differential channels
#    the ADC range is -5 to +5 V

    set sel [expr ($ch - 1) + 0x18]
    set CSR [expr 0x2160 + $sel]

    write-register Control $CSR
}

proc read_HallProbe {} {
   global mname current16ADCchannel

    set mname IP16ADC

#    first adc channel is the Hall Probe

    set current16ADCchannel 1
    select_adc_channel
#    force adc to convert
    write-register Convert -1

#    and read it

    set adc [expr int(([read-register Data] - 32768) * (5000.0/32768))]
    return $adc
}

proc read_FaradayCup {} {
   global mname current16ADCchannel

    set mname IP16ADC

#    second adc channel is the Faraday Cup

    set current16ADCchannel 2
    select_adc_channel
#    force adc to convert
    write-register Convert -1

#    and read it

    set adc [expr int(([read-register Data] - 32768) * (5000.0/32768))]
    return $adc
}

proc monitor_adcs {} {
    global mname current16ADCchannel
    global NumSamples
    global faradaycup hallprobe
    global faradaycupvisible hallprobevisible
    upvar #0 preferences(Monitor.Samples) Samples

    set adc [read_HallProbe]

    set hallprobe [expr $hallprobe + $adc]
    if {$Samples == $NumSamples} then {
#insert-debug "hallprobe 1 $adc"
        set adc [expr $hallprobe/$Samples]
        :modify HALLPROBE :value $adc
        if {$hallprobevisible == 1} then {
#insert-debug "hallprobe 2 $adc"
            :trace_append "HallProbeTrace" $adc
            :trace_update "HallProbeTrace"
        }
        set hallprobe 0
    }

    set adc [read_FaradayCup]

    set faradaycup [expr $faradaycup + $adc]
    if {$Samples == $NumSamples} then {
#insert-debug "faraday cup 1 $adc"
       set adc [expr $faradaycup/$Samples]
       :modify FARADAYCUP :value $adc
       if {$faradaycupvisible == 1} then {
#insert-debug "faraday cup 2 $adc"
            :trace_append "FaradayCupTrace" $adc
            :trace_update "FaradayCupTrace"
       }
       set faradaycup 0
    }
}

proc set-dac {i v} {
    global mname currentDACchannel

    set mname IPDAC

    set currentDACchannel $i
    set vl [expr int(($v * (4095/100.0)) + 0.5)]
    write-register c$currentDACchannel.DAC $vl
}

proc set_VoltageOutput {v}     {
    upvar #0 preferences(Voltage.p1) p1
    upvar #0 preferences(Voltage.p2) p2
#    value in physical units => convert to % of range
    set-dac 1 [expr ($p1 * $v) + $p2]
    :modify DAC1 :value $v
}

proc set_CurrentLimit {v}     {
    upvar #0 preferences(Current.p1) p1
    upvar #0 preferences(Current.p2) p2
#    value in physical units => convert to % of range
    set-dac 2 [expr ($p1 * $v) + $p2]
    :modify DAC2 :value $v
}

proc set_Injection {v}     {
    upvar #0 preferences(Injection.p1) p1
    upvar #0 preferences(Injection.p2) p2
#    value in Amps => convert to % of range
    set-dac 3 [expr ($p1 * $v) + $p2]
    :modify DAC3 :value $v
}

proc set_Extraction {v}     {
    upvar #0 preferences(Extraction.p1) p1
    upvar #0 preferences(Extraction.p2) p2
#    value in Amps => convert to % of range
    set-dac 5 [expr ($p1 * $v) + $p2]
    :modify DAC5 :value $v
}

proc set_Middle {v}     {
    upvar #0 preferences(Middle.p1) p1
    upvar #0 preferences(Middle.p2) p2
#    value in Amps => convert to % of range
    set-dac 4 [expr ($p1 * $v) + $p2]
    :modify DAC4 :value $v
}

proc read-dac {i} {
    global mname currentDACchannel

    set mname IPDAC

    set currentDACchannel $i
    set vl [expr [read-register c$currentDACchannel.DAC] / (4095/100.0)]
    return $vl
}

proc read_VoltageOutput     {} {
    upvar #0 preferences(Voltage.p1) p1
    upvar #0 preferences(Voltage.p2) p2
    return [expr ([read-dac 1] - $p2) / $p1]
}

proc read_CurrentLimit     {} {
    upvar #0 preferences(Current.p1) p1
    upvar #0 preferences(Current.p2) p2
    return [expr ([read-dac 2] - $p2) / $p1]
}

proc read_Injection     {} {
    upvar #0 preferences(Injection.p1) p1
    upvar #0 preferences(Injection.p2) p2
    return [expr ([read-dac 3] - $p2) / $p1]
}

proc read_Extraction     {} {
    upvar #0 preferences(Extraction.p1) p1
    upvar #0 preferences(Extraction.p2) p2
    return [expr ([read-dac 5] - $p2) / $p1]
}

proc read_Middle     {} {
    upvar #0 preferences(Middle.p1) p1
    upvar #0 preferences(Middle.p2) p2
    return [expr ([read-dac 4] - $p2) / $p1]
}

proc click-update {} {

    :modify DAC1 :value [string range [read_VoltageOutput] 0 4]
    :modify DAC2 :value [string range [read_CurrentLimit] 0 4]
    :modify DAC3 :value [string range [read_Injection] 0 4]
    :modify DAC4 :value [string range [read_Middle] 0 4]
    :modify DAC5 :value [string range [read_Extraction] 0 4]
    :modify HALLPROBE  :value [string range [read_HallProbe] 0 4]
    :modify FARADAYCUP :value [string range [read_FaradayCup] 0 4]
}

proc click-monitor {} {
    global autorefresh
    
    if {$autorefresh == 0} then {
        :modify MONITOR :label "Stop Monitor"
        set autorefresh 1
        run-monitor
    } else {
        :modify MONITOR :label "Start Monitor"
        set autorefresh 0
    }
}

proc run-monitor {} {
    global autorefresh autosuspend EGreport
    global NumSamples
    upvar #0 preferences(Monitor.Samples) Samples
    upvar #0 preferences(Monitor.Period) Period

    if {$autorefresh == 0} {return}
    if {$autosuspend == 0} then {
       set EGreport_SAVE $EGreport
       set EGreport -1
       incr NumSamples
       monitor_adcs
       if {$Samples == $NumSamples} then {set NumSamples 0}
       set EGreport $EGreport_SAVE
    }
    after $Period run-monitor
}

proc start-hallprobe {} {
  global hallprobevisible
  global autosuspend
  upvar #0 preferences(HallProbe.Trace.Width) w
  upvar #0 preferences(HallProbe.Trace.Heigth) h
  upvar #0 preferences(HallProbe.Trace.Ymin) ymin
  upvar #0 preferences(HallProbe.Trace.Ymax) ymax
  upvar #0 preferences(HallProbe.Trace.Xrange) xmax

LoadPreferences

   if {$hallprobevisible == 0} then {
       set autosuspend 1
       :trace "HallProbeTrace" :label "Hall Probe" :w $w :h $h :xmin 0 :xmax $xmax :ymin $ymin :ymax $ymax
       set hallprobevisible 1
       set autosuspend 0
   } else {
       open-frame "HallProbeTrace"
   }
}
   
proc start-faradaycup {} {
  global faradaycupvisible
  global autosuspend
  upvar #0 preferences(FaradayCup.Trace.Width) w
  upvar #0 preferences(FaradayCup.Trace.Heigth) h
  upvar #0 preferences(FaradayCup.Trace.Ymin) ymin
  upvar #0 preferences(FaradayCup.Trace.Ymax) ymax
  upvar #0 preferences(FaradayCup.Trace.Xrange) xmax

LoadPreferences

   if {$faradaycupvisible == 0} then {
       set autosuspend 1
       :trace "FaradayCupTrace" :label "Faraday Cup" :w $w :h $h :xmin 0 :xmax $xmax :ymin $ymin :ymax $ymax
       set faradaycupvisible 1
       set autosuspend 0
   } else {
       open-frame "FaradayCupTrace"
   }
}

proc do-reload {} {
  global autorefresh faradaycupvisible hallprobevisible

    set autorefresh 0

    LoadPreferences
    make-frame
    set faradaycupvisible 0
    set hallprobevisible 0
    click-update
}

proc do_scan {} {
    global scanning
    upvar #0 preferences(Scan.Current.Min) cmin
    upvar #0 preferences(Scan.Current.Max) cmax
    upvar #0 preferences(Scan.Current.Step) cstep
    upvar #0 preferences(Scan.Current.Settle) cset
    upvar #0 preferences(Scan.Voltage.Min) vmin
    upvar #0 preferences(Scan.Voltage.Max) vmax
    upvar #0 preferences(Scan.Voltage.Step) vstep
    upvar #0 preferences(Scan.Voltage.Settle) vset
    upvar #0 preferences(Scan.Injection.Min) imin
    upvar #0 preferences(Scan.Injection.Max) imax
    upvar #0 preferences(Scan.Injection.Step) istep
    upvar #0 preferences(Scan.Injection.Settle) iset
    upvar #0 preferences(Scan.Middle.Min) mmin
    upvar #0 preferences(Scan.Middle.Max) mmax
    upvar #0 preferences(Scan.Middle.Step) mstep
    upvar #0 preferences(Scan.Middle.Settle) mset
    upvar #0 preferences(Scan.Extraction.Min) emin
    upvar #0 preferences(Scan.Extraction.Max) emax
    upvar #0 preferences(Scan.Extraction.Step) estep
    upvar #0 preferences(Scan.Extraction.Settle) eset
    global ScanMode ScanModes

LoadPreferences

#    check for abort request
    if {$scanning != 0} {
       set r [midas-confirmation "Do you want to abort the current scan?"]
       if {$r == 1} then {set scanning -1}
       return
    }
    
    set w [string range [lindex $ScanModes $ScanMode] 0 0]

#    calcualate the time this will take
    switch $w {
      V    {set time [expr (($vmax - $vmin) / $vstep) * $vset]}
      C    {set time [expr (($cmax - $cmin) / $cstep) * $cset]}
      I    {set time [expr (($imax - $imin) / $istep) * $iset]}
      M    {set time [expr (($mmax - $mmin) / $mstep) * $mset]}
      E    {set time [expr (($emax - $emin) / $estep) * $eset]}
    }

    set r [midas-confirmation "With the current parameters this scan will take $time seconds.\n Are you sure to wish to continue?"]
    if {$r != 1} then {return}

#    get final confirmation

    switch $w {
      V    {set r [midas-confirmation "The Voltage Output will be scanned from [set vmin]% to [set vmax]%.\n Start now?"]}
      C    {set r [midas-confirmation "The Current Limit will be scanned from [set cmin] Gauss to [set cmax] Gauss.\n Start now?"]}
      I    {set r [midas-confirmation "The Injection DAC will be scanned from [set imin] Amps to [set imax] Amps.\n Start now?"]}
      M    {set r [midas-confirmation "The Middle DAC will be scanned from [set mmin] Amps to [set mmax] Amps.\n Start now?"]}
      E    {set r [midas-confirmation "The Extraction DAC will be scanned from [set emin] Amps to [set emax] Amps.\n Start now?"]}
    }
     if {$r != 1} then {return}

#    disable Start Scan button and Change to Stop
    :modify SCAN :label "Stop Scan"

#    perform the scan

    set scanning 1
    switch $w {
       V   {scan_voltage}
       C   {scan_current}
       I   {scan_injection}
       M   {scan_middle}
       E   {scan_extraction}
    }
    set scanning 0

#    finished - restore button
     :modify SCAN :label "Start Scan"
}

proc scan_current {} {
    global EGreport scanning ScanData
    upvar #0 preferences(Scan.Current.Min) min
    upvar #0 preferences(Scan.Current.Max) max
    upvar #0 preferences(Scan.Current.Step) step
    upvar #0 preferences(Scan.Current.Settle) set
    upvar #0 preferences(Scan.Current.Init) iset
    upvar #0 preferences(Scan.SaveDirectory)  sdir

   upvar #0 browserdirectory directory
   upvar #0 browserfile file

    log "Initialise current"

    set_CurrentLimit $min
    update
    after [expr int ($iset * 1000)]
    
    set voltage [read_VoltageOutput]
    set current [read_CurrentLimit]
    set injection [read_Injection]
    set middle [read_Middle]
    set extraction [read_Extraction]

    log "Starting current scan"

    set EGreport -1

    set current $min
    set i 0

    while {$current <= $max} {
        incr i
        set_CurrentLimit $current
        update
        after [expr int ($set * 1000)]
        if {$scanning != 1} then {log "Scan aborted"; return}
        set ScanData(Current,$i) $current
        set ScanData(Voltage,$i) $voltage
        set ScanData(Injection,$i) $injection
        set ScanData(Middle,$i) $middle
        set ScanData(Extraction,$i) $extraction
        set ScanData(HallProbe,$i) [read_HallProbe]
        :modify HALLPROBE :value $ScanData(HallProbe,$i)
        set ScanData(FaradayCup,$i) [read_FaradayCup]
        :modify FARADAYCUP :value $ScanData(FaradayCup,$i)
        set current [expr $current + $step]
    }

    log "Scan completed - now saving data"

#    scan completed - save the information

    set directory $sdir
    set file "Scan-[date +%Y-%h-%d-%H-%M-%S]"

    midas-browser W save_scan $i

}

proc scan_voltage {} {
    global EGreport scanning ScanData
    upvar #0 preferences(Scan.Voltage.Min) min
    upvar #0 preferences(Scan.Voltage.Max) max
    upvar #0 preferences(Scan.Voltage.Step) step
    upvar #0 preferences(Scan.Voltage.Settle) set
    upvar #0 preferences(Scan.Voltage.Init) iset
    upvar #0 preferences(Scan.SaveDirectory)  sdir

   upvar #0 browserdirectory directory
   upvar #0 browserfile file

    log "Initialise voltage"

    set_VoltageOutput $min
    update
    after [expr int ($iset * 1000)]
    
    set voltage [read_VoltageOutput]
    set current [read_CurrentLimit]
    set injection [read_Injection]
    set middle [read_Middle]
    set extraction [read_Extraction]

    log "Starting voltage scan"

    set EGreport -1

    set voltage $min
    set i 0

    while {$voltage <= $max} {
        incr i
        set_VoltageOutput $voltage
        update
        after [expr int ($set * 1000)]
        if {$scanning != 1} then {log "Scan aborted"; return}
        set ScanData(Current,$i) $current
        set ScanData(Voltage,$i) $voltage
        set ScanData(Injection,$i) $injection
        set ScanData(Middle,$i) $middle
        set ScanData(Extraction,$i) $extraction
        set ScanData(HallProbe,$i) [read_HallProbe]
        :modify HALLPROBE :value $ScanData(HallProbe,$i)
        set ScanData(FaradayCup,$i) [read_FaradayCup]
        :modify FARADAYCUP :value $ScanData(FaradayCup,$i)
        set voltage [expr $voltage + $step]
    }

    log "Scan completed - now saving data"

#    scan completed - save the information

    set directory $sdir
    set file "Scan-[date +%Y-%h-%d-%H-%M-%S]"

    midas-browser W save_scan $i

}

proc scan_injection {} {
    global EGreport scanning ScanData
    upvar #0 preferences(Scan.Injection.Min) min
    upvar #0 preferences(Scan.Injection.Max) max
    upvar #0 preferences(Scan.Injection.Step) step
    upvar #0 preferences(Scan.Injection.Settle) set
    upvar #0 preferences(Scan.Injection.Init) iset
    upvar #0 preferences(Scan.SaveDirectory)  sdir

   upvar #0 browserdirectory directory
   upvar #0 browserfile file

    log "Initialise injection"

    set_Injection $min
    update
    after [expr int ($iset * 1000)]
    
    set voltage [read_VoltageOutput]
    set current [read_CurrentLimit]
    set injection [read_Injection]
    set middle [read_Middle]
    set extraction [read_Extraction]

    log "Starting injection scan"

    set EGreport -1

    set injection $min
    set i 0

    while {$injection <= $max} {
        incr i
        set_Injection $injection
        update
        after [expr int ($set * 1000)]
        if {$scanning != 1} then {log "Scan aborted"; return}
        set ScanData(Current,$i) $current
        set ScanData(Voltage,$i) $voltage
        set ScanData(Injection,$i) $injection
        set ScanData(Middle,$i) $middle
        set ScanData(Extraction,$i) $extraction
        set ScanData(HallProbe,$i) [read_HallProbe]
        :modify HALLPROBE :value $ScanData(HallProbe,$i)
        set ScanData(FaradayCup,$i) [read_FaradayCup]
        :modify FARADAYCUP :value $ScanData(FaradayCup,$i)
        set voltage [expr $voltage + $step]
    }

    log "Scan completed - now saving data"

#    scan completed - save the information

    set directory $sdir
    set file "Scan-[date +%Y-%h-%d-%H-%M-%S]"

    midas-browser W save_scan $i

}

proc scan_extraction {} {
    global EGreport scanning ScanData
    upvar #0 preferences(Scan.Extraction.Min) min
    upvar #0 preferences(Scan.Extraction.Max) max
    upvar #0 preferences(Scan.Extraction.Step) step
    upvar #0 preferences(Scan.Extraction.Settle) set
    upvar #0 preferences(Scan.Extraction.Init) iset
    upvar #0 preferences(Scan.SaveDirectory)  sdir

   upvar #0 browserdirectory directory
   upvar #0 browserfile file

    log "Initialise extraction"

    set_Extraction $min
    update
    after [expr int ($iset * 1000)]
    
    set voltage [read_VoltageOutput]
    set current [read_CurrentLimit]
    set injection [read_Injection]
    set middle [read_Middle]
    set extraction [read_Extraction]

    log "Starting extraction scan"

    set EGreport -1

    set extraction $min
    set i 0

    while {$extraction <= $max} {
        incr i
        set_Extraction $extraction
        update
        after [expr int ($set * 1000)]
        if {$scanning != 1} then {log "Scan aborted"; return}
        set ScanData(Current,$i) $current
        set ScanData(Voltage,$i) $voltage
        set ScanData(Injection,$i) $injection
        set ScanData(Middle,$i) $middle
        set ScanData(Extraction,$i) $extraction
        set ScanData(HallProbe,$i) [read_HallProbe]
        :modify HALLPROBE :value $ScanData(HallProbe,$i)
        set ScanData(FaradayCup,$i) [read_FaradayCup]
        :modify FARADAYCUP :value $ScanData(FaradayCup,$i)
        set voltage [expr $voltage + $step]
    }

    log "Scan completed - now saving data"

#    scan completed - save the information

    set directory $sdir
    set file "Scan-[date +%Y-%h-%d-%H-%M-%S]"

    midas-browser W save_scan $i

}

proc scan_middle {} {
    global EGreport scanning ScanData
    upvar #0 preferences(Scan.Middle.Min) min
    upvar #0 preferences(Scan.Middle.Max) max
    upvar #0 preferences(Scan.Middle.Step) step
    upvar #0 preferences(Scan.Middle.Settle) set
    upvar #0 preferences(Scan.Middle.Init) iset
    upvar #0 preferences(Scan.SaveDirectory)  sdir

   upvar #0 browserdirectory directory
   upvar #0 browserfile file

    log "Initialise middle"

    set_Middle $min
    update
    after [expr int ($iset * 1000)]
    
    set voltage [read_VoltageOutput]
    set current [read_CurrentLimit]
    set injection [read_Injection]
    set middle [read_Middle]
    set extraction [read_Extraction]

    log "Starting middle scan"

    set EGreport -1

    set middle $min
    set i 0

    while {$middle <= $max} {
        incr i
        set_Middle $middle
        update
        after [expr int ($set * 1000)]
        if {$scanning != 1} then {log "Scan aborted"; return}
        set ScanData(Current,$i) $current
        set ScanData(Voltage,$i) $voltage
        set ScanData(Injection,$i) $injection
        set ScanData(Middle,$i) $middle
        set ScanData(Extraction,$i) $extraction
        set ScanData(HallProbe,$i) [read_HallProbe]
        :modify HALLPROBE :value $ScanData(HallProbe,$i)
        set ScanData(FaradayCup,$i) [read_FaradayCup]
        :modify FARADAYCUP :value $ScanData(FaradayCup,$i)
        set voltage [expr $voltage + $step]
    }

    log "Scan completed - now saving data"

#    scan completed - save the information

    set directory $sdir
    set file "Scan-[date +%Y-%h-%d-%H-%M-%S]"

    midas-browser W save_scan $i

}

proc save_scan {n} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file
   global ScanData

    set z [catch {open [file join $directory $file] w} f]
    if {$z != 0} then {midas-warning "Unable to write [file join $directory $file]\n$f"; return}

      for {set i 1} {$i <= $n} {incr i} {
        puts $f [format "%-8s  %-8s  %-8s  %-8s  %-8s  %-8s  %-8s" \
           $ScanData(Current,$i) $ScanData(Voltage,$i) $ScanData(Injection,$i) $ScanData(Middle,$i) $ScanData(Extraction,$i) $ScanData(HallProbe,$i) $ScanData(FaradayCup,$i)]
      }
    close $f

   log "Scan saved to [file join $directory $file]"
   log "Display requested"

   display_data $n
}

proc display_data {n} {
   upvar #0 browserdirectory directory
   upvar #0 browserfile file
   global ScanData

##
    set z [catch {open [file join $directory [set file]-FaradayCup] w} f]
    if {$z != 0} then {midas-warning "Unable to write [file join $directory [set file]-FaradayCup]\n$f"; return}

      for {set i 1} {$i <= $n} {incr i} {
        puts $f [format "%-12s" $ScanData(FaradayCup,$i)]
      }
    close $f

    exec ascii-to-eg -from [file join $directory [set file]-FaradayCup] -to [file join $directory [set file]-FaradayCup.msf] -format y
##
    set z [catch {open [file join $directory [set file]-HallProbe] w} f]
    if {$z != 0} then {midas-warning "Unable to write [file join $directory [set file]-HallProbe]\n$f"; return}

      for {set i 1} {$i <= $n} {incr i} {
        puts $f [format "%-12s" $ScanData(HallProbe,$i)]
      }
    close $f

    exec ascii-to-eg -from [file join $directory [set file]-Hallprobe] -to [file join $directory [set file]-HallProbe.msf] -format y
##

    for {set i 1} {$i <= $n} {incr i} {
        set hp $ScanData(HallProbe,$i)
        set fc $ScanData(FaradayCup,$i)
        if {[info exists d($hp)]} then {
            if {$fc > $d($hp)} then {set d($hp) $fc}
        } else {
            set d($hp) $fc
        }
    }
    
    set z [catch {open [file join $directory [set file]-FCvHP] w} f]
    if {$z != 0} then {midas-warning "Unable to write [file join $directory [set file]-FCvHP]\n$f"; return}
    
    set k 0
    for {set i 100} {$i <= 1500} {incr i} {
        if {[info exists d($i)]} then {puts $f "$i $d($i)"; set k $d($i)} else {puts $f "$i $k"}
    }
    
    close $f

    exec ascii-to-eg -from [file join $directory [set file]-FCvHP] -to [file join $directory [set file]-FCvHP.msf] -format xy
##

    inform sd show [list [file join $directory [set file]-FaradayCup.msf] [file join $directory [set file]-HallProbe.msf] \
          [file join $directory [set file]-FCvHP.msf]]
}


proc click arg {
   global debug expertlist
   global ScanMode

   if {$debug} then {insert-debug "click with argument \{$arg\}"}
   set f [lindex $arg 0]
   set c [lindex $arg 1]
   set w [lindex $arg 2]
   set v [lindex $arg 3]
   set i [lindex $arg 4]
   if {$c==":quit"} then {click-quit; return}
   switch -glob $w {

      DIAGNOSTICS {
          switch $v {
            IP-DAC        {inform ipdac   "Start 1"}
            IP-16ADC      {inform ip16adc "Start 2"}
          }
      }

      DAC1    {set_VoltageOutput $v}
      DAC2    {set_CurrentLimit  $v}
      DAC3    {set_Injection $v}
      DAC4    {set_Middle $v}
      DAC5    {set_Extraction $v}

      SSCAN   {set ScanMode $v}
      SCAN    {do_scan}

      MONITOR  {click-monitor}

      EXPERT  {
         set action [lindex $expertlist $i]
         set r 1
         if {$i > 0} then {
             set r [midas-confirmation "You are about to execute the action \"[first $action]\"\nAre you sure?"]
         }
         if {$r == 1} then {eval [second $action]}
      }

      default   {puts stdout $arg}
   }
}

proc make-frame {} {
  global env expertlist
  upvar #0 preferences(List.Rows) ListRows
  upvar #0 preferences(List.Cols) ListCols
  upvar #0 preferences(HallProbe.Trace.Ymin) hpmin
  upvar #0 preferences(HallProbe.Trace.Ymax) hpmax
  upvar #0 preferences(Voltage.Min)  vmin
  upvar #0 preferences(Voltage.Max)  vmax
  upvar #0 preferences(Voltage.Resolution)  vres
  upvar #0 preferences(Current.Min)  cmin
  upvar #0 preferences(Current.Max)  cmax
  upvar #0 preferences(Current.Resolution)  cres
  upvar #0 preferences(Injection.Min)  imin
  upvar #0 preferences(Injection.Max)  imax
  upvar #0 preferences(Injection.Resolution)  ires
  upvar #0 preferences(Middle.Min)  mmin
  upvar #0 preferences(Middle.Max)  mmax
  upvar #0 preferences(Middle.Resolution)  mres
  upvar #0 preferences(Extraction.Min)  emin
  upvar #0 preferences(Extraction.Max)  emax
  upvar #0 preferences(Extraction.Resolution)  eres
  upvar #0 preferences(FaradayCup.Trace.Ymin) fcmin
  upvar #0 preferences(FaradayCup.Trace.Ymax) fcmax
  global ScanMode ScanModes

LoadPreferences

  :frame [self frame-name] :label "[self frame-title]" :show-footer T \
      :icon-image "@$env(ICONHOME)/default" \
      :icon-name "[self icon-title]" :resize F

   :panel DACS :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 20

      :hskip 300
      :message DATETIME :label "[date]" :bold T
      :next-row T 

      :slider DAC1 :label "Voltage (%)"             :min $vmin :max $vmax  :valuex 20c  :resolution $vres
      :next-row T
      :slider DAC2 :label "Current (Gauss)"         :min $cmin :max $cmax  :valuex 20c  :resolution $cres
      :next-row T
      :slider DAC3 :label "Injection (Amps)"        :min $imin :max $imax  :valuex 20c  :resolution $ires
      :next-row T
      :slider DAC4 :label "Middle (Amps)"           :min $mmin :max $mmax  :valuex 20c  :resolution $mres
      :next-row T
      :slider DAC5 :label "Extraction (Amps)"       :min $emin :max $emax  :valuex 20c  :resolution $eres
      :next-row T
      :next-row T
      :slider HALLPROBE :label "Hall Probe(mV)"  :min $hpmin :max $hpmax :deaf T :valuex 20c
      :button B1 :label "Show Hall Probe History" :command start-hallprobe :w 24
      :next-row T
      :slider FARADAYCUP :label "Faraday Cup(mV)"  :min $fcmin :max $fcmax :deaf T :valuex 20c
      :button B2 :label "Show Faraday Cup History" :command start-faradaycup :w 24
      :next-row T
      :button MONITOR :label "Start Monitor" 
      :choice SSCAN  :label "DAC to scan" :strings $ScanModes :value $ScanMode
      :button SCAN   :label "Start Scan"

   :panel LOG :layout V
    :galley LIST :rows $ListRows :cols $ListCols :deaf T  :font variable

    :modify LIST :createtag "alarm  -foreground red"
    :modify LIST :createtag "log -foreground white"
    :modify LIST :createtag "warning -foreground yellow"
    :modify LIST :createtag "footer -foreground white"
   
   :panel BUTTONS  :layout T :padx 10 :pady 10
      :layout H; :row-gap 5; :col-gap 10

      :button UPDATE        :command click-update
      :button CLEAR         :label "Empty Text Window"           :command ":modify LIST :empty T"
      :button INITIALISE    :label "Initialise Control System"   :command "initialise"

      set menu {}
      foreach item $expertlist {lappend menu [first $item]}
      :menubutton EXPERT      :strings "$menu"

      :menubutton DIAGNOSTICS :strings [list IP-DAC IP-16ADC]

   :show T
}

proc update_datetime {} {
    :modify DATETIME :label "[date]"
    after 1000 update_datetime
}

proc LoadPreferences {} {
   global preferences preferencelist
   
   load_preferences

   foreach item $preferencelist {
       set p [first $item]
       set v [second $item]

       if {![info exists preferences($p)]} then {
           set preferences($p) $v
       }
   }

   update_preferences
}

proc start args {
   global started
   if {$started == 0} then {
      LoadPreferences
      make-frame
      set started 1
      update_datetime
   } else {open-frame}
}

proc set-footer {s} {:modify LIST :usingtag footer  :strings "$s\n"}
proc log {s}        {:modify LIST :usingtag log     :strings "[date] $s\n"}
proc alarm {s}      {:modify LIST :usingtag alarm   :strings "[date] $s\n"}
proc warning {s}    {:modify LIST :usingtag warning :strings "[date] $s\n"}


proc :trace_update {frame} {
  upvar #0 __[set frame]_trace trace
  upvar #0 __[set frame]_name name
  global _winname _canvas

  set xc $name(xoffset)
  set yc $name(yoffset)

  set f [lowercase $frame]
  if {[string match .* $f] != 0} then {set f [string range $f 1 end]}
  set _canvas   $_winname($f).trace

   if {![winfo ismapped .$f]} then {return}

  :draw modify :tag t :delete T

  set i 0
  foreach vl $trace {
  
    set x [expr 1.0*($i/$name(xscale)) + $name(xoffset)]
    set y [expr (0 - 1.0*($vl/$name(yscale))) + $name(yoffset)]
    
    :draw line :tag t :colour red :value $xc $yc $x $y

    set xc $x
    set yc $y
    
    incr i
  }
}
 
proc :trace_append {frame vl} {
    upvar #0 __[set frame]_trace trace
    set trace [lreplace $trace 0 0]
    lappend trace $vl
}

proc :trace args {
###########
###########
  global env

#    :trace name :label ? :icon-name ? :icon-image ? :w ? :h ? :toplevel T|F
#        :win-ctrl N|F|T  :xmin  :xmax  :ymin  :ymax

#    some defaults

  set xw 0; set xh 0
  set type "T"; set transient "F"; set xmin 0; set xmax 250; set ymin 0; set ymax 250

  switch $env(platform) {
     windows  {set winc N}
     unix     {set winc F}
     default  {set winc T}
  }
  
  set frame [lindex $args 0]
  set fl "$frame"
  set in "$frame"
  set ii "@$env(ICONHOME)/default"
  
  upvar #0 __[set frame]_name name
  upvar #0 __[set frame]_trace trace
  
  set trace {}
  for {set i 0} {$i < $xmax} {incr i} {lappend trace 0}

  set i 1
  while {$i < [llength $args]} {
    set p [lindex $args $i]
    set v [lindex $args [expr $i + 1]]
    switch $p {

      :w          {set xw $v}
      :h          {set xh $v}
      :label      {set ll "$v"}
      :icon-name  {set in "$v"}
      :icon-image {set ii "$v"}
      :toplevel   {set type "$v"}
      :win-ctrl   {set winc "$v"}
      :transient  {set transient "$v"}
      :xmin       {set xmin $v}
      :xmax       {set xmax $v}
      :ymin       {set ymin $v}
      :ymax       {set ymax $v}
      
      default {puts stderr ":frame - unknown arg $p"}
    }
    set i [expr $i + 2]
  }
  
  set name(xscale) [expr 1.0*($xmax - $xmin)/$xw]
  set name(yscale) [expr 1.0*($ymax - $ymin)/$xh]
  set name(xoffset) 10
  set name(yoffset) [expr ($ymax/$name(yscale)) + 10]

  :frame $frame :label "$fl" :icon-name "$in" :icon-image "$ii" :toplevel $type :transient $transient :win-ctrl $winc
#    :panel P1
    
    :canvas TRACE  :w [expr $xw + 20] :h [expr $xh + 20]
     [:path TRACE] configure -background black
        
    :draw text :tag m0 :delete T :colour white :text $ymin :value 10 [expr $xh + 10]
    :draw text :tag m1 :delete T :colour white :text 0     :value $name(xoffset) $name(yoffset)
    :draw text :tag m2 :delete T :colour white :text $ymax :value 10 10
    
    :draw line :tag l0 :delete T :colour green :value $name(xoffset) $name(yoffset) [expr $xw + $name(xoffset)] $name(yoffset)

   :show T
}

#proc write-register {args} {insert-debug "write: $args"}
#set rrdata 0
#proc read-register {args} {
#  global rrdata
#  set rrdata [expr rand()]
#  set vl [expr int ($rrdata * 65536)]
#  insert-debug "read: $args  $vl"
#  return $vl
#}
